
// ************************ BEGIN_COPYRIGHT_NOTICE ************************
//
// This source code copyright (c) 1995 by Paul Keister and Eric Dill
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that this copyright notice appear in all copies
// and that the author's name not be used in advertising or publicity
// pertaining to distribution of the software without specific, written
// prior permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.
//
// Please send all bug fixes, ideas, and comments to:
//
//                                      CompuServe 71242,1145 / 75061,3310
//
// ************************* END_COPYRIGHT_NOTICE *************************

// =>RESOURCE REQUIREMENTS
//
// Because this version of the ptrStream library uses the Borland
// OWL printing classes, the resource file \include\owl\printer.rc
// must be included in your build
//

#ifndef __PRTSTREAM__H
#define __PRTSTREAM__H


//
// provided for compatibily with BC++ 4.0x
//

#if __BORLANDC__ < 0x460
   #define  bool  BOOL
   #define  false FALSE
   #define  true  TRUE
#endif


#include <strstream.h>
#include <iomanip.h>
#include <string.h>

#include <owl\module.h>
#include <owl\dc.h>
#include <owl\printer.h>


#define  ESC         ( (char) 0x1b )
#define  LMARGIN_W   6


#define WHITESP      " "
#define MaxIndex     10
#define defTopM       0.5
#define defLeftM      0.5
#define defPgLen     11.0
#define defPrtLimit  defPgLen-defTopM

//
// Special effects objects
//

enum SpecialEffect
   {
      Box,           Frame,
      BitMap,        Font,
      LeftMargin,    TopMargin,
      SetPrintPos,   SaveState,     RestoreState,
      PageBreak,     CondPageBreak,
      StartHeader,   EndHeader,     PrtPageNum,
      StartFooter,   EndFooter,
      LastEffect
   };


enum prtStreamState
   {
      LeftMargin_,   TopMargin_, PrintPos_
   };


//
// prtStream class defininition
//

class prtStream : public ostrstream
{
   //
   // margins expressed in inches externally, 0.001 inches internally
   // all positions are relative to upper left corner of printable area
   //

   public:

      //
      // constructor / destructor
      //

      prtStream( float leftMargin = defLeftM, float topMargin = defTopM );
      virtual ~prtStream( void );


      //
      // public member functions for general use
      //

      void     Print( TWindow* currentWin, TPrinter& printer );


      //
      // These functions are for use by the SpecialEffect<..>::Render()
      // functions and would be private with friend templates
      //

      void  PrintLine(     void );
      void  PrintLine(     const char *buff );
      void  NewPage(       void );
      void  SaveState(     prtStreamState state = PrintPos_, int index = 0 );
      void  RestoreState(  prtStreamState state = PrintPos_, int index = 0 );
      void  SetPrintPos(   float y, float x = -1, bool ignoreMargin = false );
      void  SetFont(       LOGFONT& lf );

      TPrintDC&   PDC(        void ) { return *pdc;     }

      const char *Cursor(     void ) { return cursor;   }
      const char *TooFar(     void ) { return tooFar;   }
      const int   yPrintPos(  void ) { return printPos; }
      const int   xPrintPos(  void ) { TPoint pos;
                                       pdc->GetCurrentPosition( pos );
                                       return pos.x;
                                     }

      const int xScale(       void ) { return xSCALE; }
      const int yScale(       void ) { return ySCALE; }

      const int   PageNum(    void ) { return pageNum; }
      const float TopMargin(  void ) { return float(topMargin)  / ySCALE; }
      const float LeftMargin( void ) { return float(leftMargin) / xSCALE; }
      const int   LeftMargin( int  ) { return leftMargin; }

      void        LeftMargin( float margin )  { leftMargin = margin * xSCALE; }
      void        TopMargin(  float margin )  { topMargin  = margin * ySCALE;
                                                if ( float(printPos) / ySCALE < margin )
                                                   SetPrintPos( margin, 0 );
                                              }

      void        HeaderStart( const char *ptr ) { headerStart = ptr; }
      void        HeaderEnd(   const char *ptr ) { headerEnd   = ptr; }
      void        FooterStart( const char *ptr ) { footerStart = ptr; }
      void        FooterEnd(   const char *ptr ) { footerEnd   = ptr; }


   private:
      TEXTMETRIC  tm;
      TFont      *Font;
      TWindow    *Parent;
      TPrintDC   *pdc;
      float       initLeftMargin,
                  initTopMargin;
      int         xSCALE,                 // units per inch along the x-axis
                  ySCALE,                 // units per inch along the y-axis
                  printPos,
                  leftMargin,
                  topMargin,
                  lineLen,
                  pageNum,
                  savedPos[MaxIndex+1];   // should be dynamically sized
      bool        newLine,
                  newPage;
      const char *curLine,
                 *cursor,
                 *tooFar,
                 *headerStart,  *headerEnd,
                 *footerStart,  *footerEnd;

      void   ConfigureMapMode( void );
      void   SpecialEffect(    void );
      void   PrintPage( int page = 0 );

      void   ReadCursor( void );
      void   Scan( const char *start, const char *pastEnd );

      friend class prtStreamPage;


    public:

      //
      // dispatch table for special effect functions
      //    would be private with friend templates
      //

      void (*RenderFunctions[ LastEffect ])( prtStream *pclass, istream& in );
};


#include "prtManip.h"

#endif //  __PRTSTREAM__H

